﻿using Microsoft.AspNetCore.Mvc.ApplicationModels;
using Microsoft.Extensions.DependencyInjection;
using OpenHarbor.CQRS.Abstractions.Discovery;
using System;

namespace OpenHarbor.CQRS.AspNetCore.Mvc;

public class QueryControllerConvention : IControllerModelConvention
{
    private readonly IServiceProvider _serviceProvider;

    public QueryControllerConvention(IServiceProvider serviceProvider)
    {
        _serviceProvider = serviceProvider;
    }

    public void Apply(ControllerModel controller)
    {
        if (controller.ControllerType.IsGenericType && controller.ControllerType.Name.Contains("QueryController") && controller.ControllerType.Assembly == typeof(QueryControllerConvention).Assembly)
        {
            var genericType = controller.ControllerType.GenericTypeArguments[0];
            var queryDiscovery = _serviceProvider.GetRequiredService<IQueryDiscovery>();
            var query = queryDiscovery.FindQuery(genericType);
            controller.ControllerName = query.LowerCamelCaseName;
        }
    }
}