﻿using System.Collections.Generic;
using System.Reflection;
using Microsoft.AspNetCore.Mvc.ApplicationParts;
using Microsoft.AspNetCore.Mvc.Controllers;
using Microsoft.Extensions.DependencyInjection;
using OpenHarbor.CQRS.Abstractions.Discovery;
using OpenHarbor.CQRS.AspNetCore.Abstractions.Attributes;
using OpenHarbor.CQRS.DynamicQuery.Discover;

namespace OpenHarbor.CQRS.DynamicQuery.AspNetCore.Mvc;

public class DynamicQueryControllerFeatureProvider : IApplicationFeatureProvider<ControllerFeature>
{
    private readonly ServiceProvider _serviceProvider;

    public DynamicQueryControllerFeatureProvider(ServiceProvider serviceProvider)
    {
        _serviceProvider = serviceProvider;
    }

    public void PopulateFeature(IEnumerable<ApplicationPart> parts, ControllerFeature feature)
    {
        var queryDiscovery = _serviceProvider.GetRequiredService<IQueryDiscovery>();
        foreach (var queryMeta in queryDiscovery.GetQueries())
        {
            var ignoreAttribute = queryMeta.QueryType.GetCustomAttribute<QueryControllerIgnoreAttribute>();
            if (ignoreAttribute != null)
                continue;

            if (queryMeta.Category != "DynamicQuery")
                continue;

            if (queryMeta is DynamicQueryMeta dynamicQueryMeta)
            {
                if (dynamicQueryMeta.ParamsType == null)
                {
                    var controllerType = typeof(DynamicQueryController<,>).MakeGenericType(queryMeta.QueryType, dynamicQueryMeta.SourceType, dynamicQueryMeta.DestinationType);
                    var controllerTypeInfo = controllerType.GetTypeInfo();
                    feature.Controllers.Add(controllerTypeInfo);
                }
                else
                {
                    var controllerType = typeof(DynamicQueryController<,,>).MakeGenericType(queryMeta.QueryType, dynamicQueryMeta.SourceType, dynamicQueryMeta.DestinationType, dynamicQueryMeta.ParamsType);
                    var controllerTypeInfo = controllerType.GetTypeInfo();
                    feature.Controllers.Add(controllerTypeInfo);
                }
            }
        }
    }
}
